"""ARP Runtime API facade (preferred) + low-level client package."""

from __future__ import annotations

from importlib import import_module
from typing import TYPE_CHECKING, Any

__all__ = [
    'ArpApiError',
    'RuntimeClient',
]

_EXPORT_MAP: dict[str, str] = {
    'RuntimeClient': '.facade',
}

if TYPE_CHECKING:
    from arp_standard_client.errors import ArpApiError
    from .facade import RuntimeClient

def __getattr__(name: str) -> Any:
    if name == "ArpApiError":
        from arp_standard_client.errors import ArpApiError as _ArpApiError

        return _ArpApiError
    module = _EXPORT_MAP.get(name)
    if module is None:
        raise AttributeError(name)
    if module.startswith("."):
        return getattr(import_module(module, __name__), name)
    return getattr(import_module(module), name)




from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from arp_standard_model import ErrorEnvelope
from arp_standard_model import RunListResponse
from ...types import UNSET, Unset
from typing import cast



def _get_kwargs(
    *,
    page_size: int | Unset = UNSET,
    page_token: str | Unset = UNSET,

) -> dict[str, Any]:
    

    

    params: dict[str, Any] = {}

    params["page_size"] = page_size

    params["page_token"] = page_token


    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}


    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/runs",
        "params": params,
    }


    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> ErrorEnvelope | RunListResponse:
    if response.status_code == 200:
        response_200 = RunListResponse.model_validate(response.json())



        return response_200

    response_default = ErrorEnvelope.model_validate(response.json())



    return response_default



def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[ErrorEnvelope | RunListResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    page_size: int | Unset = UNSET,
    page_token: str | Unset = UNSET,

) -> Response[ErrorEnvelope | RunListResponse]:
    """ List runs

    Args:
        page_size (int | Unset):
        page_token (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorEnvelope | RunListResponse]
     """


    kwargs = _get_kwargs(
        page_size=page_size,
page_token=page_token,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    page_size: int | Unset = UNSET,
    page_token: str | Unset = UNSET,

) -> ErrorEnvelope | RunListResponse | None:
    """ List runs

    Args:
        page_size (int | Unset):
        page_token (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorEnvelope | RunListResponse
     """


    return sync_detailed(
        client=client,
page_size=page_size,
page_token=page_token,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    page_size: int | Unset = UNSET,
    page_token: str | Unset = UNSET,

) -> Response[ErrorEnvelope | RunListResponse]:
    """ List runs

    Args:
        page_size (int | Unset):
        page_token (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorEnvelope | RunListResponse]
     """


    kwargs = _get_kwargs(
        page_size=page_size,
page_token=page_token,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    page_size: int | Unset = UNSET,
    page_token: str | Unset = UNSET,

) -> ErrorEnvelope | RunListResponse | None:
    """ List runs

    Args:
        page_size (int | Unset):
        page_token (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorEnvelope | RunListResponse
     """


    return (await asyncio_detailed(
        client=client,
page_size=page_size,
page_token=page_token,

    )).parsed



from __future__ import annotations

from .daemon import AuthenticatedClient as DaemonAuthenticatedClient
from .daemon import Client as DaemonClient
from .runtime import AuthenticatedClient as RuntimeAuthenticatedClient
from .runtime import Client as RuntimeClient
from .tool_registry import AuthenticatedClient as ToolRegistryAuthenticatedClient
from .tool_registry import Client as ToolRegistryClient

__all__ = [
    "DaemonAuthenticatedClient",
    "DaemonClient",
    "RuntimeAuthenticatedClient",
    "RuntimeClient",
    "ToolRegistryAuthenticatedClient",
    "ToolRegistryClient",
]



from __future__ import annotations

from typing import Any


class ArpApiError(RuntimeError):
    def __init__(
        self,
        code: str,
        message: str,
        *,
        details: Any | None = None,
        status_code: int | None = None,
        raw: Any | None = None,
    ) -> None:
        super().__init__(message)
        self.code = code
        self.message = message
        self.details = details
        self.status_code = status_code
        self.raw = raw

    def __str__(self) -> str:
        if self.status_code is None:
            return f"{self.code}: {self.message}"
        return f"[{self.status_code}] {self.code}: {self.message}"


__all__ = ["ArpApiError"]



from __future__ import annotations

__all__ = [
    "__version__",
    "SPEC_REF",
    "clients",  # pyright: ignore[reportUnsupportedDunderAll]
    "daemon",  # pyright: ignore[reportUnsupportedDunderAll]
    "errors",  # pyright: ignore[reportUnsupportedDunderAll]
    "models",  # pyright: ignore[reportUnsupportedDunderAll]
    "runtime",  # pyright: ignore[reportUnsupportedDunderAll]
    "tool_registry",  # pyright: ignore[reportUnsupportedDunderAll]
    "ArpApiError",
]

__version__ = "0.2.2"
SPEC_REF = "spec/v1@v0.2.2"

from .errors import ArpApiError  # noqa: E402



from __future__ import annotations

from typing import Any

import httpx

from arp_standard_client.errors import ArpApiError
from arp_standard_model import (
    DaemonCreateInstancesRequest,
    DaemonDeleteInstanceRequest,
    DaemonDeleteRuntimeProfileRequest,
    DaemonGetRunResultRequest,
    DaemonGetRunStatusRequest,
    DaemonGetRunTraceRequest,
    DaemonHealthRequest,
    DaemonListInstancesRequest,
    DaemonListRunsRequest,
    DaemonListRuntimeProfilesRequest,
    DaemonRegisterInstanceRequest,
    DaemonSubmitRunRequest,
    DaemonUpsertRuntimeProfileRequest,
    DaemonVersionRequest,
    Health,
    InstanceCreateRequest,
    InstanceCreateResponse,
    InstanceListResponse,
    InstanceRegisterRequest,
    InstanceRegisterResponse,
    RunListResponse,
    RunRequest,
    RunResult,
    RunStatus,
    RuntimeProfile,
    RuntimeProfileListResponse,
    RuntimeProfileUpsertRequest,
    TraceResponse,
    VersionInfo,
)

from .api.health import (
    health,
)

from .api.instances import (
    create_instances,
    delete_instance,
    list_instances,
    register_instance,
)

from .api.runs import (
    get_run_result,
    get_run_status,
    get_run_trace,
    list_runs,
    submit_run,
)

from .api.runtime_profiles import (
    delete_runtime_profile,
    list_runtime_profiles,
    upsert_runtime_profile,
)

from .api.version import (
    version,
)

from .client import Client as _LowLevelClient
from arp_standard_model import ErrorEnvelope as _ErrorEnvelope
from .types import Response as _Response
from .types import Unset as _Unset
from .types import UNSET as _UNSET

def _raise_for_error_envelope(*, envelope: _ErrorEnvelope, status_code: int | None, raw: Any | None) -> None:
    details: Any | None = None
    if not isinstance(envelope.error.details, _Unset):
        details = envelope.error.details
    raise ArpApiError(
        code=str(envelope.error.code),
        message=str(envelope.error.message),
        details=details,
        status_code=status_code,
        raw=raw,
    )

def _unwrap(response: _Response[Any], *, allow_none: bool = False) -> Any:
    parsed = response.parsed
    if parsed is None:
        if allow_none:
            return None
        raise ArpApiError(
            code="unexpected_empty_response",
            message="API returned an empty response",
            status_code=int(response.status_code),
            raw=response.content,
        )
    if isinstance(parsed, _ErrorEnvelope):
        _raise_for_error_envelope(
            envelope=parsed,
            status_code=int(response.status_code),
            raw=parsed.model_dump(exclude_none=True),
        )
    return parsed

class DaemonClient:
    def __init__(
        self,
        base_url: str | None = None,
        *,
        client: _LowLevelClient | None = None,
        timeout: httpx.Timeout | None = None,
        headers: dict[str, str] | None = None,
        cookies: dict[str, str] | None = None,
        verify_ssl: Any = True,
        follow_redirects: bool = False,
        raise_on_unexpected_status: bool = False,
        httpx_args: dict[str, Any] | None = None,
    ) -> None:
        if client is None:
            if base_url is None:
                raise ValueError("base_url is required when client is not provided")
            client = _LowLevelClient(
                base_url=base_url,
                timeout=timeout,
                headers={} if headers is None else dict(headers),
                cookies={} if cookies is None else dict(cookies),
                verify_ssl=verify_ssl,
                follow_redirects=follow_redirects,
                raise_on_unexpected_status=raise_on_unexpected_status,
                httpx_args={} if httpx_args is None else dict(httpx_args),
            )
        self._client = client

    @property
    def raw_client(self) -> _LowLevelClient:
        return self._client

    def create_instances(self, request: DaemonCreateInstancesRequest) -> InstanceCreateResponse:
        resp = create_instances.sync_detailed(client=self._client, body=request.body)
        return _unwrap(resp)

    def delete_instance(self, request: DaemonDeleteInstanceRequest) -> None:
        params = request.params
        resp = delete_instance.sync_detailed(client=self._client, instance_id=params.instance_id)
        _unwrap(resp, allow_none=True)
        return None

    def delete_runtime_profile(self, request: DaemonDeleteRuntimeProfileRequest) -> None:
        params = request.params
        resp = delete_runtime_profile.sync_detailed(client=self._client, runtime_profile=params.runtime_profile)
        _unwrap(resp, allow_none=True)
        return None

    def get_run_result(self, request: DaemonGetRunResultRequest) -> RunResult:
        params = request.params
        resp = get_run_result.sync_detailed(client=self._client, run_id=params.run_id)
        return _unwrap(resp)

    def get_run_status(self, request: DaemonGetRunStatusRequest) -> RunStatus:
        params = request.params
        resp = get_run_status.sync_detailed(client=self._client, run_id=params.run_id)
        return _unwrap(resp)

    def get_run_trace(self, request: DaemonGetRunTraceRequest) -> TraceResponse:
        params = request.params
        resp = get_run_trace.sync_detailed(client=self._client, run_id=params.run_id)
        return _unwrap(resp)

    def health(self, request: DaemonHealthRequest) -> Health:
        _ = request
        resp = health.sync_detailed(client=self._client)
        return _unwrap(resp)

    def list_instances(self, request: DaemonListInstancesRequest) -> InstanceListResponse:
        _ = request
        resp = list_instances.sync_detailed(client=self._client)
        return _unwrap(resp)

    def list_runs(self, request: DaemonListRunsRequest) -> RunListResponse:
        params = request.params
        resp = list_runs.sync_detailed(client=self._client, page_size=_UNSET if params.page_size is None else params.page_size, page_token=_UNSET if params.page_token is None else params.page_token)
        return _unwrap(resp)

    def list_runtime_profiles(self, request: DaemonListRuntimeProfilesRequest) -> RuntimeProfileListResponse:
        _ = request
        resp = list_runtime_profiles.sync_detailed(client=self._client)
        return _unwrap(resp)

    def register_instance(self, request: DaemonRegisterInstanceRequest) -> InstanceRegisterResponse:
        resp = register_instance.sync_detailed(client=self._client, body=request.body)
        return _unwrap(resp)

    def submit_run(self, request: DaemonSubmitRunRequest) -> RunStatus:
        resp = submit_run.sync_detailed(client=self._client, body=request.body)
        return _unwrap(resp)

    def upsert_runtime_profile(self, request: DaemonUpsertRuntimeProfileRequest) -> RuntimeProfile:
        params = request.params
        resp = upsert_runtime_profile.sync_detailed(client=self._client, runtime_profile=params.runtime_profile, body=request.body)
        return _unwrap(resp)

    def version(self, request: DaemonVersionRequest) -> VersionInfo:
        _ = request
        resp = version.sync_detailed(client=self._client)
        return _unwrap(resp)

__all__ = [
    'DaemonClient',
]



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from arp_standard_model import ErrorEnvelope
from arp_standard_model import VersionInfo
from typing import cast



def _get_kwargs(
    
) -> dict[str, Any]:
    

    

    

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/version",
    }


    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> ErrorEnvelope | VersionInfo:
    if response.status_code == 200:
        response_200 = VersionInfo.model_validate(response.json())



        return response_200

    response_default = ErrorEnvelope.model_validate(response.json())



    return response_default



def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[ErrorEnvelope | VersionInfo]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,

) -> Response[ErrorEnvelope | VersionInfo]:
    """ Version

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorEnvelope | VersionInfo]
     """


    kwargs = _get_kwargs(
        
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,

) -> ErrorEnvelope | VersionInfo | None:
    """ Version

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorEnvelope | VersionInfo
     """


    return sync_detailed(
        client=client,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,

) -> Response[ErrorEnvelope | VersionInfo]:
    """ Version

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorEnvelope | VersionInfo]
     """


    kwargs = _get_kwargs(
        
    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,

) -> ErrorEnvelope | VersionInfo | None:
    """ Version

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorEnvelope | VersionInfo
     """


    return (await asyncio_detailed(
        client=client,

    )).parsed
